@@ -32,7 +32,7 @@
 #include "internal.h"
 #include "rtjpeg.h"
 
-typedef struct {
+typedef struct NuvContext {
     AVFrame *pic;
     int codec_frameheader;
     int quality;
@@ -74,9 +74,12 @@ static const uint8_t fallback_cquant[] = {
  */
 static void copy_frame(AVFrame *f, const uint8_t *src, int width, int height)
 {
-    AVPicture pic;
-    avpicture_fill(&pic, src, AV_PIX_FMT_YUV420P, width, height);
-    av_picture_copy((AVPicture *)f, &pic, AV_PIX_FMT_YUV420P, width, height);
+    uint8_t *src_data[4];
+    int src_linesize[4];
+    av_image_fill_arrays(src_data, src_linesize, src,
+                         f->format, width, height, 1);
+    av_image_copy(f->data, f->linesize, src_data, src_linesize,
+                  f->format, width, height);
 }
 
 /**
@@ -128,7 +131,7 @@ static int codec_reinit(AVCodecContext *avctx, int width, int height,
         avctx->height = c->height = height;
         ptr = av_fast_realloc(c->decomp_buf, &c->decomp_size,
                               c->height * c->width * 3 / 2 +
-                              FF_INPUT_BUFFER_PADDING_SIZE +
+                              AV_INPUT_BUFFER_PADDING_SIZE +
                               RTJPEG_HEADER_SIZE);
         if (!ptr) {
             av_log(avctx, AV_LOG_ERROR,
@@ -171,7 +174,7 @@ static int decode_frame(AVCodecContext *avctx, void *data, int *got_frame,
     // codec data (rtjpeg quant tables)
     if (buf[0] == 'D' && buf[1] == 'R') {
         int ret;
-        // skip rest of the frameheader.
+        // Skip the rest of the frame header.
         buf       = &buf[12];
         buf_size -= 12;
         ret       = get_quant(avctx, c, buf, buf_size);
@@ -198,11 +201,11 @@ static int decode_frame(AVCodecContext *avctx, void *data, int *got_frame,
         keyframe = 1;
         break;
     }
-    // skip rest of the frameheader.
+    // Skip the rest of the frame header.
     buf       = &buf[12];
     buf_size -= 12;
     if (comptype == NUV_RTJPEG_IN_LZO || comptype == NUV_LZO) {
-        int outlen = c->decomp_size - FF_INPUT_BUFFER_PADDING_SIZE;
+        int outlen = c->decomp_size - AV_INPUT_BUFFER_PADDING_SIZE;
         int inlen  = buf_size;
         if (av_lzo1x_decode(c->decomp_buf, &outlen, buf, &inlen)) {
             av_log(avctx, AV_LOG_ERROR, "error during lzo decompression\n");
@@ -332,5 +335,5 @@ AVCodec ff_nuv_decoder = {
     .init           = decode_init,
     .close          = decode_end,
     .decode         = decode_frame,
-    .capabilities   = CODEC_CAP_DR1,
+    .capabilities   = AV_CODEC_CAP_DR1,
 };
