@@ -32,22 +32,19 @@
 #include <stdlib.h>
 
 #include "libavutil/channel_layout.h"
-#include "avcodec.h"
-#include "internal.h"
-#include "mathops.h"
 
 #define BITSTREAM_READER_LE
-#include "get_bits.h"
+#include "avcodec.h"
 #include "bytestream.h"
+#include "get_bits.h"
+#include "internal.h"
+#include "mathops.h"
 
 #define SMKTREE_BITS 9
 #define SMK_NODE 0x80000000
 #define SMKTREE_DECODE_MAX_RECURSION 32
 #define SMKTREE_DECODE_BIG_MAX_RECURSION 500
 
-/*
- * Decoder context
- */
 typedef struct SmackVContext {
     AVCodecContext *avctx;
     AVFrame *pic;
@@ -100,8 +97,8 @@ enum SmkBlockTypes {
 static int smacker_decode_tree(GetBitContext *gb, HuffContext *hc, uint32_t prefix, int length)
 {
     if (length > SMKTREE_DECODE_MAX_RECURSION) {
-        av_log(NULL, AV_LOG_ERROR, "Maximum tree recursion level exceeded.\n");
-        return AVERROR_INVALIDDATA;
+      av_log(NULL, AV_LOG_ERROR, "Maximum tree recursion level exceeded.\n");
+      return AVERROR_INVALIDDATA;
     }
 
     if(!get_bits1(gb)){ //Leaf
@@ -545,12 +542,6 @@ static int decode_frame(AVCodecContext *avctx, void *data, int *got_frame,
 }
 
 
-
-/*
- *
- * Uninit smacker decoder
- *
- */
 static av_cold int decode_end(AVCodecContext *avctx)
 {
     SmackVContext * const smk = avctx->priv_data;
@@ -566,11 +557,6 @@ static av_cold int decode_end(AVCodecContext *avctx)
 }
 
 
-/*
- *
- * Init smacker decoder
- *
- */
 static av_cold int decode_init(AVCodecContext *avctx)
 {
     SmackVContext * const c = avctx->priv_data;
@@ -782,7 +768,7 @@ AVCodec ff_smacker_decoder = {
     .init           = decode_init,
     .close          = decode_end,
     .decode         = decode_frame,
-    .capabilities   = CODEC_CAP_DR1,
+    .capabilities   = AV_CODEC_CAP_DR1,
 };
 
 AVCodec ff_smackaud_decoder = {
@@ -792,5 +778,5 @@ AVCodec ff_smackaud_decoder = {
     .id             = AV_CODEC_ID_SMACKAUDIO,
     .init           = smka_decode_init,
     .decode         = smka_decode_frame,
-    .capabilities   = CODEC_CAP_DR1,
+    .capabilities   = AV_CODEC_CAP_DR1,
 };
