@@ -1,6 +1,6 @@
 /*
  * Matroska constants
- * Copyright (c) 2003-2004 The ffmpeg Project
+ * Copyright (c) 2003-2004 The FFmpeg project
  *
  * This file is part of Libav.
  *
@@ -113,6 +113,7 @@
 #define MATROSKA_ID_VIDEOPIXELCROPR 0x54DD
 #define MATROSKA_ID_VIDEODISPLAYUNIT 0x54B2
 #define MATROSKA_ID_VIDEOFLAGINTERLACED 0x9A
+#define MATROSKA_ID_VIDEOFIELDORDER 0x9D
 #define MATROSKA_ID_VIDEOSTEREOMODE 0x53B8
 #define MATROSKA_ID_VIDEOASPECTRATIO 0x54B3
 #define MATROSKA_ID_VIDEOCOLORSPACE 0x2EB524
@@ -221,6 +222,21 @@ typedef enum {
   MATROSKA_TRACK_ENCODING_COMP_HEADERSTRIP = 3,
 } MatroskaTrackEncodingCompAlgo;
 
+typedef enum {
+    MATROSKA_VIDEO_INTERLACE_FLAG_UNDETERMINED = 0,
+    MATROSKA_VIDEO_INTERLACE_FLAG_INTERLACED   = 1,
+    MATROSKA_VIDEO_INTERLACE_FLAG_PROGRESSIVE  = 2
+} MatroskaVideoInterlaceFlag;
+
+typedef enum {
+    MATROSKA_VIDEO_FIELDORDER_PROGRESSIVE  = 0,
+    MATROSKA_VIDEO_FIELDORDER_UNDETERMINED = 2,
+    MATROSKA_VIDEO_FIELDORDER_TT           = 1,
+    MATROSKA_VIDEO_FIELDORDER_BB           = 6,
+    MATROSKA_VIDEO_FIELDORDER_TB           = 9,
+    MATROSKA_VIDEO_FIELDORDER_BT           = 14,
+} MatroskaVideoFieldOrder;
+
 typedef enum {
   MATROSKA_VIDEO_STEREOMODE_TYPE_MONO               = 0,
   MATROSKA_VIDEO_STEREOMODE_TYPE_LEFT_RIGHT         = 1,
@@ -254,6 +270,7 @@ typedef struct CodecTags{
 
 extern const CodecTags ff_mkv_codec_tags[];
 extern const CodecMime ff_mkv_mime_tags[];
+extern const CodecMime ff_mkv_image_mime_tags[];
 extern const AVMetadataConv ff_mkv_metadata_conv[];
 
 int ff_mkv_stereo3d_conv(AVStream *st, MatroskaVideoStereoModeType stereo_mode);
